//
//  KSVPNUFacade.h
//  VPNUnlimitedSDK
//
//  Copyright (c) 2015 KeepSolid. All rights reserved.
//

//
// Use of this SDK is subject to the VPN Unlimited SDK License Agreement
// https://www.keepsolid.com/static/docs/VPN_Unlimited_SDK_License.html
//

#import <Foundation/Foundation.h>

@class VPNUTransport;
@class VPNUConfigurator;
@class VPNUAccountUserInfo;
@class VPNUAccountStatus;
@class VPNUPurchaseItem;
@class VPNUAccountDevice;
@class VPNUServerItem;
@class VPNUAccountDeviceStatistics;
@class VPNUAuthorizer;

/*!
 @class KSVPNUFacade
 @abstract
 Class which serves as a 'facade' to other VPN Unlimited SDK classes.
 @discussion
 This class hides the complexity of VPN Unlimited SDK by encapsulating most of the needed features.
 */
NS_AVAILABLE(10_7, 7_0)
@interface KSVPNUFacade : NSObject

/*!
 @brief Intance of @c VPNUAuthorizer which is used authorize to server.
 @discussion This object is read-only and created lazily inside @c KSVPNUFacade implementation.
 @see @c VPNUAuthorizer
 */
@property (readonly) VPNUAuthorizer *authorizer;

/*!
 @brief Intance of @c VPNUTransport which is used to send requests to server.
 @discussion This object is read-only and created lazily inside @c KSVPNUFacade implementation.
 @see @c VPNUTransport
 */
@property (readonly) VPNUTransport *transport;
/*!
 @brief Intance of @c VPNUConfigurator which is used to configure and control VPN connections.
 @discussion This object is read-only and created lazily inside @c KSVPNUFacade implementation.
 @see @c VPNUConfigurator
 */
@property (readonly) VPNUConfigurator *configurator;

/*!
 @brief Info about currently logged-in user.
 @see @c VPNUAccountUserInfo
 */
@property (strong) VPNUAccountUserInfo *currentUserInfo;
/*!
 @brief Currently logged-in user's account status.
 @see @c VPNUAccountStatus
 */
@property (strong) VPNUAccountStatus *currentAccountStatus;
/*! @brief Application scheme used by local server. */
@property (copy) NSString *appScheme;
/*! @brief Custom name for a downloaded VPN profile. */
@property (copy) NSString *profileName;

/*! @brief Singleton instance of Facade. */
+ (instancetype)defaultFacade;

@end

@interface KSVPNUFacade (General)

/*!
 @brief Sets the shared keychain group ID needed for deviceID storing in iOS keychain.
 @param groupID Your shared keychain group ID.
 */
- (void)setSharedKeychainGroupID:(NSString *)groupID;

/*!
 @brief Sets the application ID needed for request signing.
 @param appID Your application ID.
 */
- (void)setApplicationID:(NSString *)appID;
/*!
 @brief Sets the application secret key needed for request signing.
 @param appSecret Your application secret key.
 */
- (void)setApplicationSecret:(NSString *)appSecret;
/*!
 @brief Sets the application version.
 @param applicationVersion Your application version.
 */
- (void)setApplicationVersion:(NSString *)appVersion;

@end

@interface KSVPNUFacade (Networking)

/*!
 @brief TBD
 */
- (void)geolocateMeWithCompletion:(void(^)(NSError *error, NSDictionary *result))completion;
/*!
 @brief Retieves a list of available VPN servers for currently logged-in user.
 @param completion Block to be called upon operation completion.
 @see @c VPNUServerItem
 */
- (void)serversListWithCompletion:(void(^)(NSError *error, NSArray<VPNUServerItem *> *serversList))completion;
/*!
 @brief Retieves a list of available VPN servers with no need to logged-in.
 @param completion Block to be called upon operation completion.
 @see @c VPNUServerItem
 */
- (void)freeServersListWithCompletion:(void(^)(NSError *error, NSArray<VPNUServerItem *> *serversList))completion;
/*!
 @brief Retieves a list of optimal VPN servers for currently logged-in user.
 @param completion Block to be called upon operation completion.
 */
- (void)optimalServersListWithCompletion:(void(^)(NSError *error, NSArray<VPNUServerItem *> *serversList))completion;
/*!
 @brief Retieves a list of optimal VPN servers with no need to logged-in.
 @param completion Block to be called upon operation completion.
 @see @c VPNUServerItem
 */
- (void)freeOptimalServersListWithCompletion:(void(^)(NSError *error, NSArray<VPNUServerItem *> *serversList))completion;

/*!
 @brief Validates a provided purchase item on server.
 @param purchaseItem A purchase item for which to perform the validation.
 @param completion Block to be called upon operation completion.
 @see VPNUPurchaseItem
 */
- (void)validatePurchaseItem:(VPNUPurchaseItem *)purchaseItem
                  completion:(void(^)(NSError *error))completion;
/*!
 @brief Sends redeem code to server.
 @attention This method does not perform code validation locally. If you want code validation feature you have to implement it.
 @param code Reddem code string.
 @param completion Block to be called upon operation completion.
 */
- (void)enterCode:(NSString *)code completion:(void(^)(NSError *error))completion;
/*!
 @brief Retrieves the currently logged-in user's account information.
 @param completion Block to be called upon operation completion.
 @see @c VPNUAccountUserInfo
 */
- (void)userInfoWithCompletion:(void(^)(NSError *error, VPNUAccountUserInfo *userInfo))completion;
/*!
 @brief Retrieves the currently logged-in user's account status.
 @param completion Block to be called upon operation completion.
 @see VPNUAccountStatus
 */
- (void)accountStatusWithCompletion:(void(^)(NSError *error, VPNUAccountStatus *accountStatus))completion;
/*!
 @brief Retrieves a list of currently logged-in user's used devices.
 @param completion Block to be called upon operation completion.
 @see VPNUAccountDevice
 */
- (void)accountDevicesWithCompletion:(void(^)(NSError *error, NSArray<VPNUAccountDevice *> *accountDevices))completion;
/*!
 @brief Retrieves a list of currently logged-in user's statistics for provided device.
 @param device A device object for which to retrieve statistics.
 @param completion Block to be called upon operation completion.
 @see @c VPNUAccountDeviceStatistics
 */
- (void)accountStatisticsForDevice:(VPNUAccountDevice *)device
                        completion:(void(^)(NSError *error, NSArray<VPNUAccountDeviceStatistics *> *accountStatistics))completion;
/*!
 @brief Changes password for a user identifiied by @c login parameter.
 @discussion If password was changed successfully it automatically reauthorizes user with new password.
 @param login User's login/email.
 @param oldPassword User's current password.
 @param newPassword User's new password.
 @param completion Block to be called upon operation completion.
 @attention This method doesn't perform any local checks on provided passwords. If you need these checks you have to implement them.
 @see @c authorizeWithLogin:andPassword:completion:
 */
- (void)changeAccountPasswordWithLogin:(NSString *)login
                           oldPassword:(NSString *)oldPassword
                           newPassword:(NSString *)newPassword
                            completion:(void(^)(NSError *error))completion;
/*!
 @brief Performs authorization.
 @discussion This method tries to authorize a user with specified login and password. If authorization is successful it automatically sets the @c currentUserInfo property.
 @param login User's login. Cannot be @c nil.
 @param password User's password. Cannot be @c nil.
 @param completion Block to be called upon operation completion.
 */
- (void)authorizeWithLogin:(NSString *)login
               andPassword:(NSString *)password
                completion:(void(^)(NSError *error))completion;

/*!
 @brief Sends account confirmation email.
 @discussion This method sends an account confirmation letter to user's email.
 @param login User's login.
 @param completion Block to be called upon operation completion.
 */
- (void)sendConfirmationEmailWithLogin:(NSString *)login
                            completion:(void(^)(NSError *error))completion;

/*!
 @brief Registers a new user.
 @discussion This method tries to register a new user with specified login and password synchronously and waits for the response.
 If registration is successful, it automatically logs in the newly created user.
 @param login User's login. Cannot be @c nil.
 @param password User's password. Cannot be @c nil.
 @param completion Block to be called upon operation completion.
 @see @c authorizeWithLogin:andPassword:completion:
 */
- (void)registerWithLogin:(NSString *)login
              andPassword:(NSString *)password
               completion:(void(^)(NSError *error))completion;
/*!
 @brief Signs out the already authorized user.
 @discussion This method clears current session and signals server about sign-out event.
 @param completion Block to be called upon operation completion.
 */
- (void)signOutWithCompletion:(void(^)(BOOL successful, NSError *error))completion;
/*!
 @brief Sends recovery password instructions.
 @discussion Instructions are sent to the email of user specified in @c login parameter.
 @param login User's login/email.
 @param completion Block to be called upon operation completion.
 */
- (void)recoveryPasswordForLogin:(NSString *)login completion:(void(^)(NSError *error))completion;

/*!
 @brief Returns the list of available personal servers.
 @param completion Block to be called upon operation completion.
 */
- (void)personalServerRegionsList:(void(^)(NSError *error, NSDictionary *result))completion;

/*!
 @brief Returns the list of available personal IPs.
 @param completion Block to be called upon operation completion.
 */
- (void)personalIPServersList:(void(^)(NSError *error, NSDictionary *result))completion;

@end

@interface KSVPNUFacade (VPN)
/*!
 @brief Loads VPN configuration preferences.
 @discussion This method loads the preferences for VPN configuration of the current application. You can call this method if you want to retrieve @c connectionStatus property before any call to configurator.
 @param completion Block to be called upon operation completion.
 */
- (void)loadPreferencesWithCompletion:(void(^)(NSError *error))completion;
/*!
 @brief Installs VPN profile for the specified server item.
 @discussion This method first retrieves VPN profile information for the specified server item. If this step was succesful, it then tries to install a new VPN profile into the system using the information retrieved in the first step.
 @param serverItem Server item object to perform setup for. Cannot be @c nil.
 @param completion Block to be called upon operation completion. It has no return value and accepts two parameters: @c error if any error occurs and @c alreadyInstalled - set to @c YES, if VPN profile is already installed (iOS only).
 */
- (void)setupVPNWithServerItem:(VPNUServerItem *)serverItem completion:(void(^)(NSError *error, BOOL alreadyInstalled))completion;
/*!
 @brief Calls @c connectToVPNAndUseOnDemand:completion: with @c useOnDemand parameter set to @c YES.
 */
- (void)connectToVPNWithCompletion:(void(^)(NSError *error))completion;
/*!
 @brief Starts the VPN tunnel using the current VPN configuration.
 @discussion This method returns after VPN tunnel connection process is started and does not signals about the actual success of the VPN connection. In order to obtain the connection status, use the @c connectionStatus property.
 @see  @c connectionStatus
 @param useOnDemand Flag which indicates whether to automatically set On Demand value for this connection to @c YES before connecting.
 @param completion Block to be called upon operation completion.
 */
- (void)connectToVPNAndUseOnDemand:(BOOL)useOnDemand completion:(void(^)(NSError *error))completion;
/*!
 @brief Calls @c stopVPNAndUseOnDemand:completion: with @c useOnDemand parameter set to @c YES.
 */
- (void)stopVPNWithCompletion:(void(^)(NSError *error))completion;
/*!
 @brief Stops the VPN tunnel.
 @discussion This method returns after VPN tunnel disconnect process is started and does not signals about the actual success of the VPN connection. In order to obtain the connection status, use the @c connectionStatus property.
 @see  @c connectionStatus
 @param useOnDemand Flag which indicates whether to automatically set On Demand value for this connection to @c NO before disconnecting.
 @param completion Block to be called upon operation completion.
 */
- (void)stopVPNAndUseOnDemand:(BOOL)useOnDemand completion:(void(^)(NSError *error))completion;
/*!
 @brief Turns VPN On Demand on or off.
 @param enabled Boolean value which controls VPN On Demand toggling. If set to @c YES VPN On Demand is turned on, otherwise VPN On Demand is turned off.
 @param completion Block to be called upon operation completion.
 */
- (void)enableOnDemand:(BOOL)enable completion:(void(^)(NSError *error))completion;
/*!
 @brief Sets VPN configuration for current app enabled.
 @discussion If succeeded, this method disables VPN configurations of other apps.
 @param completion Block to be called upon operation completion.
 */
- (void)enableVPNWithCompletion:(void(^)(NSError *error))completion NS_AVAILABLE(10_11, 8_0);
/*!
 @brief Removes the VPN configuration from the Network Extension preferences if present.
 @param completion Block to be called upon operation completion.
 */
- (void)removeProfileWithCompletion:(void(^)(NSError *error))completion NS_AVAILABLE(10_7, 8_0);

@end

@interface KSVPNUFacade (DNS)

/*!
 @brief TBD
 @param completion Block to be called upon operation completion.
 */
- (void)dnsBlockerGetConfigWithCompletion:(void(^)(NSError *error, NSDictionary *result))completion;
/*!
 @brief TBD
 @param completion Block to be called upon operation completion.
 */
- (void)dnsBlockerGetCategoriesWithCompletion:(void(^)(NSError *error, NSDictionary *result))completion;
/*!
 @brief TBD
 @param completion Block to be called upon operation completion.
 */
- (void)dnsBlockerAttachCategory:(NSString*)category value:(NSString *)value withCompletion:(void(^)(NSError *error, NSDictionary *result))completion;
/*!
 @brief TBD
 @param completion Block to be called upon operation completion.
 */
- (void)dnsBlockerSetConfigMask:(NSNumber*)mask withCompletion:(void(^)(NSError *error, NSDictionary *result))completion;
@end

@interface KSVPNUFacade (BlackWhiteList)

/*!
 @brief TBD
 @param completion Block to be called upon operation completion.
 */

- (void)loadBlackListItemsFrom:(NSNumber*)from to:(NSNumber*)to withComplition:(void (^)(NSError *, NSDictionary *))completion;

/*!
 @brief TBD
 @param completion Block to be called upon operation completion.
 */

- (void)loadWhiteListItemsFrom:(NSNumber*)from to:(NSNumber*)to withComplition:(void (^)(NSError *, NSDictionary *))completion;

/*!
 @brief TBD
 @param completion Block to be called upon operation completion.
 */

- (void)getProtectionListInfo:(void (^)(NSError *error, NSDictionary *result))completion;

/*!
 @brief TBD
 @param completion Block to be called upon operation completion.
 */

- (void)addDomain:(NSString *)domain toList:(NSString *)categotyID completion:(void (^)(NSError *error, NSDictionary *result))completion;

/*!
 @brief TBD
 @param completion Block to be called upon operation completion.
 */

- (void)removeDomain:(NSString *)domainID completion:(void (^)(NSError *error, NSDictionary *result))completion;

/*!
 @brief TBD
 @param completion Block to be called upon operation completion.
 */

- (void)changeListStatus:(NSString *)isOn listId:(NSString *)listId completion:(void (^)(NSError *error, NSDictionary *result))completion;

@end

@interface KSVPNUFacade (CensorshipTest)

/*!
 @brief TBD
 @param completion Block to be called upon operation completion.
 */
- (void)statGetCensorshipURLs:(void(^)(NSError *error, NSArray *result))completion;

/*!
 @brief TBD
 @param completion Block to be called upon operation completion.
 */
- (void)statSendCensorshipResults:(id)data withCompletion:(void(^)(NSError *error))completion;

/*!
 @brief TBD
 @param completion Block to be called upon operation completion.
 */
- (void)statGetCensorshipBonusInfo:(void(^)(NSError *error, NSDictionary *result))completion;

@end

@interface KSVPNUFacade (SquareFlags)

/*!
 @brief TBD
 */
- (void)serversListWithSquareFlags:(BOOL)squareFlags andCompletion:(void(^)(NSError *error, NSArray<VPNUServerItem *> *serversList))completion;

/*!
 @brief TBD
 */
- (void)freeServersListWithSquareFlags:(BOOL)squareFlags andCompletion:(void(^)(NSError *error, NSArray<VPNUServerItem *> *serversList))completion;

/*!
 @brief TBD
 */
- (void)optimalServersListWithSquareFlags:(BOOL)squareFlags andCompletion:(void(^)(NSError *error, NSArray<VPNUServerItem *> *serversList))completion;

/*!
 @brief TBD
 */
- (void)freeOptimalServersListWithSquareFlags:(BOOL)squareFlags andCompletion:(void(^)(NSError *, NSArray<VPNUServerItem *> *))completion;

/*!
 @brief TBD
 */
- (void)accountStatusWithSquareFlag:(BOOL)squareFlag andCompletion:(void(^)(NSError *error, VPNUAccountStatus *accountStatus))completion;

/*!
 @brief TBD
 */
- (void)personalServerRegionsListWithSquareFlags:(BOOL)squareFlags andCompletion:(void(^)(NSError *error, NSDictionary *result))completion;

/*!
 @brief TBD
 */
- (void)personalIPServersListWithSquareFlags:(BOOL)squareFlags andCompletion:(void(^)(NSError *error, NSDictionary *result))completion;

@end
