//
//  VPNUAccount.h
//  VPNUnlimitedSDK
//
//  Copyright (c) 2015 KeepSolid. All rights reserved.
//

//
// Use of this SDK is subject to the VPN Unlimited SDK License Agreement
// https://www.keepsolid.com/static/docs/VPN_Unlimited_SDK_License.html
//


#import <Foundation/Foundation.h>

@class VPNUTransport;
@class VPNUAccountStatus;
@class VPNUAccountUserInfo;
@class VPNUAccountDeviceStatistics;
@class VPNUAccountDevice;

/*!
 @class VPNUAccount
 @abstract
 This class provides an interface for retrieving miscellaneous information about currently logged-in user.
 @discussion
 Before using this class you must authorize or register with the help of @c VPNUAuthorizer class. You must pass the same @c VPNUTransport instance to its constructor as you have passed to @c VPNUAuthorizer instance constructor.
 */

NS_AVAILABLE(10_7, 7_0)
@interface VPNUAccount : NSObject

#pragma mark - Public Methods

/*!
 @brief Retrieves the currently logged-in user's account status.
 @discussion It blocks a calling thread.
 @warning This method shouldn't be called from main thread.
 @param error If request generates an error, a new error object is created and returned as an object pointed to by this parameter. Can be @c nil.
 @return Currently logged-in user's account status.
 @see VPNUAccountStatus
 */
- (VPNUAccountStatus *)statusWithError:(NSError **)error NS_AVAILABLE(10_7, 7_0);
/*!
 @brief TBD
 */
- (VPNUAccountStatus *)statusWithSquareFlag:(BOOL)squareFlag andError:(NSError **)error NS_AVAILABLE(10_7, 7_0);
/*!
 @brief Retrieves the currently logged-in user's account information.
 @discussion It blocks a calling thread.
 @warning This method shouldn't be called from main thread.
 @param error If request generates an error, a new error object is created and returned as an object pointed to by this parameter. Can be @c nil.
 @return Currently logged-in user's account information.
 @see @c VPNUAccountUserInfo
 */
- (VPNUAccountUserInfo *)userInfoWithError:(NSError **)error NS_AVAILABLE(10_7, 7_0);
/*!
 @brief Retrieves a list of currently logged-in user's used devices.
 @discussion It blocks a calling thread.
 @warning This method shouldn't be called from main thread.
 @param error If request generates an error, a new error object is created and returned as an object pointed to by this parameter. Can be @c nil.
 @return An array of @c VPNUAccountDevice objects for currently logged-in user.
 @see VPNUAccountDevice
 */
- (NSArray<VPNUAccountDevice *> *)devicesWithError:(NSError **)error  NS_AVAILABLE(10_7, 7_0);
/*!
 @brief Retrieves a list of currently logged-in user's statistics for provided device.
 @discussion It blocks a calling thread.
 @warning This method shouldn't be called from main thread.
 @param device A device object for which to retrieve statistics.
 @param error If request generates an error, a new error object is created and returned as an object pointed to by this parameter. Can be @c nil.
 @return An array of @c VPNUAccountDeviceStatistics objects for currently logged-in user. Each object represents statistics for a specific date.
 @see VPNUAccountDeviceStatistics
 */
- (NSArray<VPNUAccountDeviceStatistics *> *)statisticsForDevice:(VPNUAccountDevice *)device withError:(NSError **)error NS_AVAILABLE(10_7, 7_0);

#pragma mark - Life Cycle
/*!
 @brief Initializes and returns a newly allocated account object with the specified transport object.
 @param transport The transport object for internal usage.
 @return An initialized account object or nil if the object couldn't be created.
 */
- (instancetype)initWithTransport:(VPNUTransport *)transport NS_AVAILABLE(10_7, 7_0) NS_DESIGNATED_INITIALIZER;

/*!
 @brief Creates and returns a newly allocated account object with the specified transport object.
 @discussion This method is a convenience constructor.
 @param transport The transport object for internal usage.
 @return An initialized account object or nil if the object couldn't be created.
 */
+ (instancetype)accountWithTransport:(VPNUTransport *)transport NS_AVAILABLE(10_7, 7_0);

@end
