//
//  VPNUAuthorizer.h
//  VPNUnlimitedSDK
//
//  Copyright (c) 2015 KeepSolid. All rights reserved.
//

//
// Use of this SDK is subject to the VPN Unlimited SDK License Agreement
// https://www.keepsolid.com/static/docs/VPN_Unlimited_SDK_License.html
//


#import <Foundation/Foundation.h>

@class VPNUTransport;
@class VPNUAccountUserInfo;

/*!
 @class VPNUAuthorizer
 @abstract
 This class provides an interface for performing various authorization-related routines.
 @discussion
 Use this class to authorize with an existing KeepSolid account or create a new one. After initializing an instance of this class with an instance of @c VPNUTransport and performing authorization, be sure to pass the same instance of @c VPNUTransport to other classes which depend on it. Otherwise you will not be able to use all features of SDK.
 */
NS_AVAILABLE(10_7, 7_0)
@interface VPNUAuthorizer : NSObject

#pragma mark - Public Methods

/*!
 @brief Performs authorization.
 @discussion This method tries to authorize a user with specified login and password synchronously and waits for the response. It blocks a calling thread.
 @warning This method shouldn't be called from main thread.
 
 @param login User's login. Cannot be @c nil.
 @param password User's password. Cannot be @c nil.
 @param error If request generates an error, a new error object is created and returned as an object pointed to by this parameter. Can be @c nil.
 @return An object which represents the user's account information if no error has occured, @c nil otherwise.
 */
- (VPNUAccountUserInfo *)authorizeWithLogin:(NSString *)login andPassword:(NSString *)password error:(NSError **)error NS_AVAILABLE(10_7, 7_0);
/*!
 @brief Registers a new user.
 @discussion This method tries to register a new user with specified login and password synchronously and waits for the response. It blocks a calling thread.
 If registration is successful, it automatically logs in the newly created user.
 @warning This method shouldn't be called from main thread.
 @param login User's login. Cannot be @c nil.
 @param password User's password. Cannot be @c nil.
 @param error If request generates an error, a new error object is created and returned as an object pointed to by this parameter. Can be @c nil.
 @return An object which represents the user's account information if no error has occured, @c nil otherwise.
 */
- (VPNUAccountUserInfo *)registerWithLogin:(NSString *)login andPassword:(NSString *)password error:(NSError **)error NS_AVAILABLE(10_7, 7_0);
/*!
 @brief Signs out the already authorized user.
 @discussion This method clears current session, signals server about sign-out event and waits for the response. It blocks a calling thread.
 @warning This method shouldn't be called from main thread.
 @param error If request generates an error, a new error object is created and returned as an object pointed to by this parameter. Can be @c nil.
 @return @c YES if no error has occured, @c NO otherwise.
 */
- (BOOL)signOutWithError:(NSError **)error NS_AVAILABLE(10_7, 7_0);
/*!
 @brief Asks the server to send confirmation email for specified login.
 @discussion This method notifies the server that it needs to resend the confirmation letter to email associated with specified login.
 @warning This method shouldn't be called from main thread.
 
 @param login User's login. Cannot be @c nil.
 @param error If request generates an error, a new error object is created and returned as an object pointed to by this parameter. Can be @c nil.
 @return @c YES if no error has occured, @c NO otherwise.
 */
- (BOOL)resendConfirmationEmailForLogin:(NSString *)login error:(NSError **)error NS_AVAILABLE(10_7, 7_0);

#pragma mark - Life Cycle

/*!
 @brief Initializes and returns a newly allocated authorizer object with the specified transport object.
 @param transport The transport object for internal usage.
 @return An initialized authorizer object or nil if the object couldn't be created.
 */
- (instancetype)initWithTransport:(VPNUTransport *)transport NS_AVAILABLE(10_7, 7_0) NS_DESIGNATED_INITIALIZER;

/*!
 @brief Creates and returns a newly allocated authorizer object with the specified transport object.
 @discussion This method is a convenience constructor.
 @param transport The transport object for internal usage.
 @return An initialized authorizer object or nil if the object couldn't be created.
 */
+ (instancetype)authorizerWithTransport:(VPNUTransport *)transport NS_AVAILABLE(10_7, 7_0);

@end
