//
//  VPNUServersListAccessor.h
//  VPNUnlimitedSDK
//
//  Copyright (c) 2015 KeepSolid. All rights reserved.
//

//
// Use of this SDK is subject to the VPN Unlimited SDK License Agreement
// https://www.keepsolid.com/static/docs/VPN_Unlimited_SDK_License.html
//

#import <Foundation/Foundation.h>

@class VPNUTransport;
@class VPNUServerItem;

/*!
 @class VPNUServersListAccessor
 @abstract
 This class provides an interface for getting the list of available VPN servers for currently logged-in user.
 @discussion
 Before using this class you must authorize or register with the help of @c VPNUAuthorizer class. You must pass the same @c VPNUTransport instance to its constructor as you have passed to @c VPNUAuthorizer instance constructor.
 */
NS_AVAILABLE(10_7, 7_0)
@interface VPNUServersListAccessor : NSObject

#pragma mark - Public Methods
/*!
 @brief Retieves a list of available VPN servers for currently logged-in user.
 @discussion It blocks a calling thread.
 @warning This method shouldn't be called from main thread.
 @param error If request generates an error, a new error object is created and returned as an object pointed to by this parameter. Can be @c nil.
 @return An array of @c VPNUServerItem objects if no error occured, @c nil otherwise.
 */
- (NSArray<VPNUServerItem *> *)listWithError:(NSError **)error NS_AVAILABLE(10_7, 7_0);

/*!
 @brief Retieves a list of available VPN servers with no need to logged-in.
 @discussion It blocks a calling thread.
 @warning This method shouldn't be called from main thread.
 @param error If request generates an error, a new error object is created and returned as an object pointed to by this parameter. Can be @c nil.
 @return An array of @c VPNUServerItem objects if no error occured, @c nil otherwise.
 */
- (NSArray<VPNUServerItem *> *)freeListWithError:(NSError **)error NS_AVAILABLE(10_7, 7_0);

/*!
 @brief Retieves a list of optimal VPN servers for currently logged-in user.
 @discussion It blocks a calling thread.
 @warning This method shouldn't be called from main thread.
 @param error If request generates an error, a new error object is created and returned as an object pointed to by this parameter. Can be @c nil.
 @return An array of @c VPNUServerItem objects if no error occured, @c nil otherwise.
 */
- (NSArray<VPNUServerItem *> *)optimalServersListWithError:(NSError **)error;

/*!
 @brief Retieves a list of optimal VPN servers with no need to logged-in.
 @discussion It blocks a calling thread.
 @warning This method shouldn't be called from main thread.
 @param error If request generates an error, a new error object is created and returned as an object pointed to by this parameter. Can be @c nil.
 @return An array of @c VPNUServerItem objects if no error occured, @c nil otherwise.
 */
- (NSArray<VPNUServerItem *> *)freeOptimalServersListWithError:(NSError **)error;

/*!
 @brief TBD
 */
@property BOOL useSquareFlags;

#pragma mark - Life Cycle

/*!
 @brief Initializes and returns a newly allocated servers list accessor object with the specified transport object.
 @param transport The transport object for internal usage.
 @return An initialized servers list accessor object or nil if the object couldn't be created.
 */
- (instancetype)initWithTransport:(VPNUTransport *)transport NS_AVAILABLE(10_7, 7_0) NS_DESIGNATED_INITIALIZER;

/*!
 @brief Creates and returns a newly allocated servers list accessor object with the specified transport object.
 @discussion This method is a convenience constructor.
 @param transport The transport object for internal usage.
 @return An initialized servers list accessor object or nil if the object couldn't be created.
 */
+ (instancetype)serversListAccessorWithTransport:(VPNUTransport *)transport NS_AVAILABLE(10_7, 7_0);

@end
