//
//  VPNUTransport.h
//  VPNUnlimitedSDK
//
//  Copyright (c) 2015 KeepSolid. All rights reserved.
//

//
// Use of this SDK is subject to the VPN Unlimited SDK License Agreement
// https://www.keepsolid.com/static/docs/VPN_Unlimited_SDK_License.html
//


#import <Foundation/Foundation.h>

@class VPNURequest;
@protocol VPNUAuthDelegateProtocol;

/*!
 @class VPNUTransport
 @abstract
 Class used to send synchronous requests to VPN Unlimited service servers.
 @discussion
 When an instance of this class is created it then can be passed as a parameter to other classes which are initialized using @c initWithtransport: method.
 
 Once sign-in request is successful, the instance of @c VPNUTransport manages the created session internally. Therefore, all subsequent requests should be made using the same instance of @c VPNUTransport.
 
 
 If you use the standard authorization routine with @c authorizeWithLogin: method of @c VPNUAuthorizer class, @c VPNUTransport silently re-authorizes if the session is expired. But if you implement non-standard authorization methods (like authorizing with social networks), it is impossible to re-authorize without calling the third-party code. In this case you must implement methods from @c VPNUAuthDelegateProtocol and pass the delegate instance to @c customAuthDelegate property before any usage of this class.
 
 @warning
 Always use the same instance of @c VPNUTransport when passing it to @c initWithTransport: methods. The class provides a special @c sharedTransport accessor method, which always returns the same global object.
 */
NS_AVAILABLE(10_7, 7_0)
@interface VPNUTransport : NSObject

/*!
 @brief A delegate object which conforms to @c VPNUAuthDelegateProtocol protocol.
 @discussion Once you define an object which conforms to @c VPNUAuthDelegateProtocol, set this property with it. If you are using non-standard authorization methods (like authorizing with social networks), you must provide an object which resolves custom authorization with third-party server and returns authentication details.
 */
@property (strong) id<VPNUAuthDelegateProtocol> customAuthDelegate;

/*!
 @brief Sends the specified request synchronously.
 @discussion This method sends a previously created instance of @c VPNURequest synchronously and waits for the response. It blocks a calling thread.
 @warning This method shouldn't be called from main thread.
 
 @param request The cunstructed request object. Cannot be @c nil.
 @param error If request generates an error, a new error object is created and returned as an object pointed to by this parameter. Can be @c nil.
 @return An arbitrary object which represents a server response if no error has occured. Can be @c nil.
 */
- (id)sendRequest:(VPNURequest *)request error:(NSError **)error NS_AVAILABLE(10_7, 7_0);

/*!
 @brief Cancels all pending requests.
 */
- (void)cancelAllRequests;

/*!
 @brief Sets the shared @c VPNUTransport instance to a specified transport object.
 @discussion An application that has special transporting requirements or constraints should use this method to specify an @c VPNUTransport instance with customized settings. The application should do so before any calls to the sharedTransport method.
 
 @param transport The transport object to use as the shared transport object.
 */
+ (void)setSharedTransport:(VPNUTransport *)transport NS_AVAILABLE(10_7, 7_0);

/*!
 @brief Returns the shared @c VPNUTransport instance.
 @discussion Applications that do not have special transporting requirements or constraints should find the default shared transport instance acceptable. An application with more specific needs can create a custom @c VPNUTransport object and set it as the shared transport instance using @c setSharedTransport:. The application should do so before any calls to this method.
 @return The shared @c VPNUTransport instance.
 */
+ (VPNUTransport *)sharedTransport NS_AVAILABLE(10_7, 7_0);

@end

